@@ -57,8 +57,8 @@ BarcodeDetector* barcodeDetector_callback; // makes it possible to callback to m
                                             // This pointer needs to be set to the correct object before calling any member functions!
 
 BarcodeDetector::BarcodeDetector() {
-    frame_width = 320;
-    frame_height = 240;
+//    frame_width = 320;
+//    frame_height = 240;
     qDebug("Init BarcodeDetector");
     // init from maemo-barcode.c
 
@@ -122,11 +122,11 @@ int BarcodeDetector::analyseImage(char *buffer, char *fourcc, int bpp) {
 //            qDebug("analyse_image(): start dmtx thread running");
 
             // copy the image data to the shared buffer
-            memcpy(this->dmtx_thread.thread_buffer, buffer, frame_width*frame_height*sizeof(char)*bpp/8); // should be the right amount of data....
+            memcpy(this->dmtx_thread.thread_buffer, buffer, frame_width*frame_height*sizeof(char)*bpp/24); // should be the right amount of data....
             this->dmtx_thread.frame_width=frame_width;
             this->dmtx_thread.frame_height=frame_height;
 
-            if(strstr((char*)fourcc, "RGB")){
+            if(strstr((char*)fourcc, "RGB3")){
                 if(bpp==16)
                     this->dmtx_thread.frame_fourcc=DmtxPack16bppRGB;
                 else if (bpp==24)
@@ -146,11 +146,10 @@ int BarcodeDetector::analyseImage(char *buffer, char *fourcc, int bpp) {
     }
     // otherwise it must be running, so leave it be
 
+    if(!this->zbar_thread.isRunning()) {
 
-
-    if(!this->zbar_thread.isRunning())
         zbar_thread.start();
-    else if(!this->zbar_thread.isDecoding){
+    } else if(!this->zbar_thread.isDecoding){
 //        qDebug() << "analyse_image() zbar thread is not decoding";
 
         // check and see if it has decoded something
@@ -168,20 +167,8 @@ int BarcodeDetector::analyseImage(char *buffer, char *fourcc, int bpp) {
             const uchar *buffer = (uchar *)this->zbar_thread.thread_buffer;
             unsigned int height = this->zbar_thread.frame_height;
             unsigned int width = this->zbar_thread.frame_width;
-            QImage::Format format = QImage::Format_Indexed8; // we can't always assume this.....
-            QImage *qi = new QImage(width, height, format); //(buffer, width, height, width, format);
-            // generate a colourmap
-            qi->setNumColors(256);
-            for(int i=0;i<256;i++){
-                qi->setColor(i,QColor(i,i,i,255).rgba());
-            }
-            qi->fill(0);
-            for(unsigned int i=0;i<width;i++){
-                for(unsigned int j=0;j<height;j++){
-                    qi->setPixel(i,j,buffer[i+j*width]);
-                }
-            }
-
+            QImage::Format format = QImage::Format_RGB888; // we can't always assume this.....
+            QImage *qi = new QImage(buffer, width, height, format); //(buffer, width, height, width, format);
 
             emit imageAnalysed(QString::fromUtf8(this->zbar_thread.barcode_type),
                                QString::fromUtf8(this->zbar_thread.thread_output),
@@ -194,7 +181,9 @@ int BarcodeDetector::analyseImage(char *buffer, char *fourcc, int bpp) {
 //            qDebug("analyse_image(): no barcode found, start zbar thread running");
 
             // copy the image data to the shared buffer
+            // set to bpp/24 for now, seems to avoid crashes..
             memcpy(this->zbar_thread.thread_buffer, buffer, frame_width*frame_height*sizeof(char)*bpp/8); // should be the right amount of data....
+
             this->zbar_thread.frame_width=frame_width;
             this->zbar_thread.frame_height=frame_height;
 
@@ -216,6 +205,15 @@ int BarcodeDetector::analyseImage(char *buffer, char *fourcc, int bpp) {
 
 
 
+}
+
+void BarcodeDetector::setFrameSize(int frameWidth, int frameHeight) {
+    this->frame_width = frameWidth;
+    this->frame_height = frameHeight;
+    this->zbar_thread.frame_width = frameWidth;
+    this->zbar_thread.frame_height = frameHeight;
+    this->dmtx_thread.frame_width = frameWidth;
+    this->dmtx_thread.frame_height = frameHeight;
 }
 
 QImage *BarcodeDetector::getLastImage() {
@@ -229,22 +227,15 @@ QImage *BarcodeDetector::getLastImage() {
     unsigned int width = this->zbar_thread.frame_width;
 
     // we can't always assume this.....
-    QImage::Format format = QImage::Format_Indexed8;
+    QImage::Format format = QImage::Format_RGB888; // should be b/w indexed 8 from zbar
 
     // we really need to do some conversions, etc. if format is not known
     // but for the time being we'll just assume it's 8bit b&w
 
     qDebug() << "getLastImage size width=" << width << " height=" << height;
 
-    QImage *qi = new QImage(width, height, format); //(buffer, width, height, width, format);
+    QImage *qi = new QImage(buffer, width, height, format); //(buffer, width, height, width, format);
 
-    // generate a colourmap
-    qi->setNumColors(256);
-    for(int i=0;i<256;i++){
-        qi->setColor(i,QColor(i,i,i,255).rgba());
-    }
-
-    qi->fill(0);
     //for(unsigned int i=0;i<height;i++){
     //    memcpy((void*)(qi->scanLine(i)), (const void*)buffer[i*width], (size_t)width);
     //}
@@ -252,13 +243,6 @@ QImage *BarcodeDetector::getLastImage() {
     //qi->fromData((const uchar *)buffer, (int)(width*height),"BMP");
     //
 
-    for(unsigned int i=0;i<width;i++){
-        for(unsigned int j=0;j<height;j++){
-            qi->setPixel(i,j,buffer[i+j*width]);
-        }
-    }
-
-
     return qi;
 }
 
